@@ -42,6 +42,11 @@ hooks:
           hostname: postgresql
           username: postgres
           password: test
+        - name: all
+          format: custom
+          hostname: postgresql
+          username: postgres
+          password: test
     mysql_databases:
         - name: test
           hostname: mysql
@@ -51,6 +56,11 @@ hooks:
           hostname: mysql
           username: root
           password: test
+        - name: all
+          format: sql
+          hostname: mysql
+          username: root
+          password: test
     mongodb_databases:
         - name: test
           hostname: mongodb
